package com.huigou.topsun.syncOrg.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSON;
import com.huigou.topsun.common.OrgKindId;
import com.huigou.topsun.syncOrg.application.OrgSyncApplication;
import com.huigou.topsun.syncOrg.vo.*;
import com.huigou.topsun.sap.common.DefaultHttpClient;
import com.huigou.topsun.sap.common.application.SapMutualEpLogApplication;
import com.huigou.topsun.sap.common.domain.EpResult;
import com.huigou.uasp.bmp.opm.domain.model.org.Org;
import com.huigou.uasp.bmp.opm.domain.model.org.Person;
import com.huigou.uasp.bmp.opm.repository.org.OrgRepository;
import com.huigou.util.DateUtil;
import com.huigou.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Auther: xin.lu
 * @Date: 2024/03/25/9:28
 * @Description:
 */
@Service("orgSyncApplication")
public class OrgSyncApplicationImpl implements OrgSyncApplication {
    private final static Logger LOG = LoggerFactory.getLogger(OrgSyncApplicationImpl.class);
    @Autowired
    private OrgRepository orgRepository;
    @Autowired
    private DefaultHttpClient httpClient;
    @Autowired
    SapMutualEpLogApplication sapMutualEpLogApplication;

    @Override
    public EpOrgVo buildOrgVo () {
        List<Org> orgList = orgRepository.findAll();
        orgList = orgList
                .stream()
                .filter(org -> org.getFullId().contains("/E3B4E5C5C6454F81921CF58B7597CE2F"))
                .collect(Collectors.toList());
        //机构、部门、岗位
        List<OrgVo> orgVoList = orgList
                .stream()
                .filter(org -> !OrgKindId.PSM.getId().equals(org.getOrgKindId()))
                .map(org -> {
                    OrgVo orgVo = new OrgVo();
                    BeanUtil.copyProperties(org,orgVo);
                    orgVo.setDelFlag("0");
                    //把id做hash处理，得到int类型数据
                    orgVo.setId(convertHashCode(org.getId()));

                    //从全路径中获取当前组织的父节点名称
                    if (StringUtil.isNotBlank(org.getParentId())){
                        String fullName = org.getFullName();
                        String[] split = fullName.split("/");
                        orgVo.setParentName(split[split.length - 2]);

                        //把父id做hash处理，得到int类型数据
                        orgVo.setParentId(convertHashCode(org.getParentId()));
                    }
                    return orgVo;
                })
                .collect(Collectors.toList());

        //人员
        List<Org> psmList = orgList
                .stream()
                .filter(org -> OrgKindId.PSM.getId().equals(org.getOrgKindId()))
                .collect(Collectors.toList());

        //先同步机构、部门、岗位、再同步组织人员
        List<OrgPersonVo> orgPersonVoList = this.buildOrgPersonVo(psmList);

        //同步人员
        List<PersonVo> personVoList = this.buildPersonVo(psmList);

        EpOrgVo epOrgVo = new EpOrgVo();
        epOrgVo.setOrgVoList(orgVoList);
        epOrgVo.setOrgPersonVoList(orgPersonVoList);
        epOrgVo.setPersonVoList(personVoList);
        LOG.info(DateUtil.getDateFormat(new Date())+"====同步的组织结构数据===={}",epOrgVo);
        return epOrgVo;
    }

    @Override
    public List<OrgPersonVo> buildOrgPersonVo(List<Org> psmList){
        return psmList
                .stream()
                .map(orgPerson -> {
                    OrgPersonVo orgPersonVo = new OrgPersonVo();
                    //组织人员id
                    orgPersonVo.setId(convertHashCode(orgPerson.getId()));
                    //部门id
                    orgPersonVo.setDeptId(convertHashCode(orgPerson.getDeptId()));
                    //岗位id
                    orgPersonVo.setPositionId(convertHashCode(orgPerson.getPositionId()));
                    //人员id
                    orgPersonVo.setPersonId(convertHashCode(orgPerson.getPerson().getId()));
                    return orgPersonVo;
                }).collect(Collectors.toList());
    }

    @Override
    public List<PersonVo> buildPersonVo(List<Org> psmList){
        return psmList
                .stream()
                .map(psm -> {
                    Person person = psm.getPerson();
                    PersonVo personVo = new PersonVo();
                    BeanUtil.copyProperties(person, personVo);
                    personVo.setDelFlag("0");
                    //把人员id做hash处理，得到int类型数据
                    personVo.setId(convertHashCode(person.getId()));
                    personVo.setDeptId(convertHashCode(psm.getDeptId()));
                    return personVo;
                }).collect(Collectors.toList());
    }

    @Override
    public List<SapPersonVo> buildSapPersonVo(){
        List<Org> orgList = orgRepository.findAll();
        //人员
        return orgList
                .stream()
                .filter(org -> OrgKindId.PSM.getId().equals(org.getOrgKindId()))
                .map(org -> {
                    SapPersonVo sapPersonVo = new SapPersonVo();
                    Person person = org.getPerson();
                    BeanUtil.copyProperties(person,sapPersonVo);
                    sapPersonVo.setParentFullName(org.getFullName());
                    return sapPersonVo;
                })
                .collect(Collectors.toList());
    }

    @Override
    public void syncOrgToSap() {

    }

    /**
     * 字符串转hashcode
     * @param value
     * @return
     */
    public String convertHashCode(String value){
        if (StringUtil.isBlank(value)){
            return "";
        }
        int abs = Math.abs(value.hashCode());
        return String.valueOf(abs);
    }

    /**
     * 记录日志
     * @param epResult
     * @param epOrgVo
     */
    public void saveMutualEpLog(EpResult epResult,EpOrgVo epOrgVo,String systemType){
        //日志记录
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("type", epResult.getCode());
        resultMap.put("message", epResult.getMsg());
        resultMap.put("businessType", systemType);
        resultMap.put("businessId", "syncOrg");
        resultMap.put("parameter", JSON.toJSONString(epOrgVo));
        resultMap.put("sequence", resultMap.get("row"));
        sapMutualEpLogApplication.saveSapMutualEpLog(resultMap);
    }
}
